@@ -1,8 +1,13 @@
 #!/usr/bin/perl -w
 use HTTP::Proxy qw( :log );
 use HTTP::Proxy::HeaderFilter::simple;
+use MIME::Base64 qw( encode_base64 );
 use strict;
 
+# the encoded user:password pair
+my $token = "Basic " . encode_base64( "http:proxy" );
+chomp $token;    # grr
+
 # a very simple proxy that requires authentication
 # login/password: http/proxy
 my $proxy = HTTP::Proxy->new;
@@ -16,8 +21,8 @@ $proxy->push_filter(
             my $auth = $self->proxy->hop_headers->header('Proxy-Authorization')
               || "";
 
-            # check the credentials (http:proxy)
-            if ( $auth ne 'Basic aHR0cDpwcm94eQ==' ) {
+            # check the credentials
+            if ( $auth ne $token ) {
                 my $response = HTTP::Response->new(407);
                 $response->header(
                     Proxy_Authenticate => 'Basic realm="HTTP::Proxy"' );
