@@ -15,13 +15,12 @@ HTTP::Proxy::BodyFilter::lines - A filter that outputs only complete lines
 
     my $filter = MyFilter->new();
 
-    # note that both filters will be run on the same messages
-    # (those with a MIME type of text/*)
+    # stack both filters so that they'll handle text/* responses
     $proxy->push_filter(
         mime     => 'text/*',
-        response => HTTP::Proxy::BodyFilter::lines->new
+        response => HTTP::Proxy::BodyFilter::lines->new,
+        response => $filter
     );
-    $proxy->push_filter( mime => 'text/*', response => $filter );
 
     # I want my lines to end with '!'
     # This is equivalent to $/ = '!' in a normal Perl program
@@ -101,6 +100,10 @@ Philippe "BooK" Bruhat, E<lt>book@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
+Copyright 2003-2004, Philippe Bruhat
+
+=head1 LICENSE
+
 This module is free software; you can redistribute it or modify it under
 the same terms as Perl itself.
 
